<?php

/**
 * @file
 * A wrapper API for the retreiving stock quotes from Google Finance.
 */

define("stockapi_trade_price_only", "1");
define("stockapi_chg", "2");
define("stockapi_pct_chg", "3");
define("stockapi_updated", "4");
define("stockapi_previous_close", "5");
define("stockapi_days_range", "6");
define("stockapi_52_week_range", "7");
define("stockapi_open", "8");
define("stockapi_volume", "9");
define("stockapi_avg_daily_volume", "10");
define("stockapi_market_cap", "11");
define("stockapi_trade_date", "12"); //string
define("stockapi_last_trade_date", "13"); //string
define("stockapi_pe_ratio", "14");
define("stockapi_float_shares", "15");
define("stockapi_chg_and_pct_chg", "16");
define("stockapi_last_trade_time", "17");
define("stockapi_exchange", "18");
define("stockapi_name", "19");
define("stockapi_vol_avg_for_postprocessing", "999");

/**
 * Implement a HTTP request to Google Finance to retrieve stock quotes.
 *
 * @param $symbols array
 *   An array of ticker symbols to send to Google. There is not limit on the
 *   the request into batches.
 *
 * @return
 *   An array of stocks with stock information from Google. False on failure.
 */
function stockapi_fetch($symbols) {
  // Step 1: Split up larger arrays.
  $stock_lookup_failure = array();
  $stock_market = '';
  $stock_results = array();
  $l = 199;
  if (count($symbols) <= $l) {
    $stock_array = $symbols;

    foreach($stock_array as $stock_symbol) {
      $stock_test = '';
      $result = db_query("SELECT exchange FROM {stockapi} WHERE symbol = :symbol", array(':symbol' => $stock_symbol))->fetchField();
      if (empty($result)) {
        $stock_test = 'TSE';
      }
      else {
        $stock_test = $result;
      }
      switch ($stock_test) {
        case 'NYSE':
          $stock_market = 'NYSE';
        break;
        case 'TSE':
          $stock_market = 'TSE';
        break;
        case 'Toronto Stock Exchange':
          $stock_market = 'TSE';
        break;
        case 'New York Stock Exchange':
          $stock_market = 'NYSE';
        break;
        case 'NASDAQ':
          $stock_market = 'NASDAQ';
        break;
        case '':
          $stock_market = 'TSE';
        break;
        case NULL:
          $stock_market = 'TSE';
        break;
        default:
          $stock_market = 'TSE';
        break;
      }
      // Expecting market:ticker example: NYSE:GM or TSE:ENB.
      //$debug_text = 'stock_test =' . $stock_test . ' , '  . $stock_market . ' = stock_market , symbol=' . $stock_symbol;
      //watchdog('stockapi_info', '<pre>@print_r</pre>', array('@print_r' =>
      //  print_r($debug_text , TRUE)
      //  ), WATCHDOG_INFO
      //);

      // Load HTML.
      $url = stockapi_google_finance_url($stock_market, $stock_symbol);
      // Scrape above web page for ticker info.
      //watchdog('stockapi_info', 'url=<pre>@print_r</pre>', array('@print_r' => print_r( $url, TRUE)));
      $page = file_get_contents($url);
      $doc = new \DOMDocument('1.0', 'UTF-8');
      // set error level
      $internalErrors = libxml_use_internal_errors(true);

      $doc->loadHTML($page);

      // Restore error level
      libxml_use_internal_errors($internalErrors);
      $content = $doc->getElementById('market-data-div');
      if (empty($content)) {
        // Try again.
        if ($stock_market == 'TSE') {
          $stock_market = 'NYSE';
        }
        if ($stock_market == 'NASDAQ') {
          $stock_market = 'TSE';
        }
        $url = stockapi_google_finance_url($stock_market, $stock_symbol);
        $page = file_get_contents($url);
        $internalErrors = libxml_use_internal_errors(true);
        // set error level
        $doc->loadHTML($page);
        // Restore error level
        libxml_use_internal_errors($internalErrors);
        $content = $doc->getElementById('market-data-div');
        if (empty($content)) {
          if ($stock_market == 'NYSE') {
            $stock_market = 'NASDAQ';
          }
          $url = stockapi_google_finance_url($stock_market, $stock_symbol);
          $page = file_get_contents($url);
          $internalErrors = libxml_use_internal_errors(true);
          // set error level
          $doc->loadHTML($page);
          // Restore error level
          libxml_use_internal_errors($internalErrors);
          if (empty($content)) {
            if ($stock_market == 'NASDAQ') {
              $stock_market = 'NYSE';
            }
            $url = stockapi_google_finance_url($stock_market, $stock_symbol);// . '&' . uniqid() . '=' . rand();
            $page = file_get_contents($url);
            $internalErrors = libxml_use_internal_errors(true);
            // set error level
            $doc->loadHTML($page);
            // Restore error level
            libxml_use_internal_errors($internalErrors);
            if (empty($content)) {
              watchdog('stockapi', 'ERR empty($content) incorrect stock_market for stock_symbol? <pre>@print_r</pre>',
               array('@print_r' => print_r( $stock_market . ':' . $stock_symbol, TRUE)
              ), WATCHDOG_ERROR);
              // Loading of page failed, skip this update.
              continue;
            }
          }
        }
      }
      $price_panel = $doc->getElementById('price-panel');
      // For NYSE / NASDAQ.
      if ($stock_market == 'NYSE' || $stock_market == 'NASDAQ') {
        $get_date = $price_panel->nodeValue;
        $sans_vide = preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n", $get_date);
        $array = explode("\n", $sans_vide);
        $after_hours = FALSE;
        foreach ($array as $key => $value) {
          if (stripos($value, 'After Hours') === 0) {
            $after_hours = TRUE;
          }
          if (stripos($value, 'PM ') > 0) {
            $get_date = $value;
          }
          if (stripos($value, 'AM ') > 0) {
            $get_date = $value;
          }
        }
        $month_day = '';
        $timeof = '';
        if ($after_hours) {
          // After normal stock trading hours.
          $date_string_array = explode(", ", $get_date);
          $month_day = trim($date_string_array[0]);
          $timeof = trim($date_string_array[1]);
          // TODO: fix when jan 1 or 2 when markets closed referring
          // To cur year instead of previous, challenging to fix though.
          date_default_timezone_set('EST');
          $stock_results[$stock_symbol][stockapi_updated] = time();
          if (stripos($timeof, 'EST') > 0) {
            $timetmp = trim($timeof);
            $timeof = substr($timetmp, 0, stripos($timeof, ' EST'));
          }
          $time_string = $month_day . ' ' . date('Y') . ' ' . $timeof;
          $time_string = trim($time_string);
          $locale_temp = Locale::getDefault();
          locale_set_default('en-US');// NASDAQ and TSE use the en_US locale for date.
          $time_updated = strtotime($time_string);
          locale_set_default($locale_temp); // Restore locale to server default.
          if (!empty($time_updated)) {
            $stock_results[$stock_symbol][stockapi_trade_date] = $time_updated;
          }
          $stock_results[$stock_symbol][stockapi_last_trade_time] = $timeof;
          $stock_results[$stock_symbol][stockapi_last_trade_date] = $month_day;
        }
        else {
          // Stock market is open.
          $localtime_assoc = localtime(time(), true);
          $min = $localtime_assoc['tm_min'];
          if (strlen($min) == 1) {
            $min = '0' . $min;
          }
          date_default_timezone_set('EST');
          $date_temp = date('M j');
          $stock_results[$stock_symbol][stockapi_updated] = time();
          $stock_results[$stock_symbol][stockapi_trade_date] = $date_temp;
          $stock_results[$stock_symbol][stockapi_last_trade_date] = $date_temp;
          $stock_results[$stock_symbol][stockapi_last_trade_time] = $localtime_assoc['tm_hour'] . 'h' . $min;
        }
      }
      $arr_spans = $price_panel->getElementsByTagName('span');
      $arr_tds = $content->getElementsByTagName('td');
      $key_tmp = '';
      $key_orig = '';
      $val_tmp = '';
      $stock_results[$stock_symbol][0] = $stock_symbol;
      $stock_results[$stock_symbol][stockapi_exchange] = $stock_market;
      //watchdog('stockapi_?', '<pre>@print_r</pre>', array('@print_r' => print_r( $stock_results, TRUE)));
      /*
       * If statement in foreach inspecing spans for price information:
       * <span class="nwp"></span> //inside is the trade date info.
       * If statement in foreach inspecing spans for change info:
       * <span class="ch bld"></span> //inside contains up or down info.
       */
      foreach($arr_spans as $el) {
        if ($el->getAttribute('class') == 'pr') {
          $key_tmp = stockapi_trade_price_only;
          $val_tmp = trim($el->nodeValue);
          $stock_results[$stock_symbol][$key_tmp] = $val_tmp;
        }
        // This section only works with TSE market.
        if ($el->getAttribute('class') == 'nwp') {
          // Span class='nwp' is Date.
          $val_tmp = trim($el->nodeValue);
          if (stripos($val_tmp, 'Close') >= 1) {
            $idx_space_minus = stripos($val_tmp, ' -');
            $date_temp = substr($val_tmp, 0, $idx_space_minus);
            // $date_temp, stores first part of date.
            if ($idx_space_minus <= 0) {
              $date_temp = substr($val_tmp, 0, 5);
              if (is_numeric(substr($val_tmp, 0, 6))) {
                $date_temp = trim($date_temp);
              }
              else {
                $date_temp = substr($val_tmp, 0, 5);
              }
            }
            $key_tmp = stockapi_updated;
            // TODO: fix when jan 1 or 2 when markets closed referring
            // To cur year instead of previous, challenging to fix though.
            $stock_results[$stock_symbol][$key_tmp] = strtotime($date_temp .
            ' ' . date('Y'));
            $stock_results[$stock_symbol][stockapi_trade_date] = $date_temp;
            /*watchdog('stockapi_date_temp', '<pre>@print_r</pre>',
               array('@print_r' => print_r( $date_temp, TRUE)));*/
            $last_trade_date = stockapi_last_trade_date;
            $stock_results[$stock_symbol][$last_trade_date] = $date_temp;
            $stock_results[$stock_symbol][stockapi_last_trade_time] = '16h00';
          }
          else {
            // Stock market is open.
            $localtime_assoc = localtime(time(), true);
            $min = $localtime_assoc['tm_min'];
            if (strlen($min) == 1) {
              $min = '0' . $min;
            }
            date_default_timezone_set('EST');
            $date_temp = date('M j');
            $stock_results[$stock_symbol][stockapi_updated] = time();
            $stock_results[$stock_symbol][stockapi_trade_date] = $date_temp;
            $stock_results[$stock_symbol][stockapi_last_trade_date] = $date_temp;
            $stock_results[$stock_symbol][stockapi_last_trade_time] = $localtime_assoc['tm_hour'] . 'h' . $min; 
          }
        }
        if ($el->getAttribute('class') == 'ch bld') {
          foreach($el->childNodes as $chld) {
            if ($chld instanceof DOMElement) {
              if (stripos($chld->nodeValue, '+') === 0 &&
                stripos($chld->nodeValue, '%') <= 0) {
                // Element $chld contains price up (positive).
                $idx_positive = stripos($chld->nodeValue, '+');
                $val_tmp = substr($chld->nodeValue, ($idx_positive+1));
                $key_tmp = stockapi_chg;
                //stockapi_chg
                $stock_results[$stock_symbol][$key_tmp] = $val_tmp;
              }
              if (stripos($chld->nodeValue, '-') === 0 &&
                stripos($chld->nodeValue, '%') <= 0) {
                // Element $chld contains text starting with - (negative)
                //stockapi_chg
                $idx_negative = stripos($chld->nodeValue, '-');
                $val_tmp = substr($chld->nodeValue, ($idx_negative));
                $key_tmp = stockapi_chg;
                $stock_results[$stock_symbol][$key_tmp] = $val_tmp;
              }
              if (stripos($chld->nodeValue, '%') >= 1 &&
                stripos($chld->nodeValue, '%') <= 8) {
                //stockapi_pct_chg
                $val_tmp = substr($chld->nodeValue, 1, 4);// Ex: '3.00'.
                $key_tmp = stockapi_pct_chg;
                $stock_results[$stock_symbol][$key_tmp] = $val_tmp;
              }
            }
          }
          if (isset($stock_results[$stock_symbol][stockapi_trade_price_only]) &&
          isset($stock_results[$stock_symbol][stockapi_chg])) {
            $prev_close = 0;
            $trade_price = $stock_results[$stock_symbol][stockapi_trade_price_only];
            $latest_chg = $stock_results[$stock_symbol][stockapi_chg];
            if ($latest_chg >= 0) {
              $prev_close = (floatval($trade_price) - $latest_chg);
            }
            else {
              $prev_close = (floatval($trade_price) + ($latest_chg*(-1)));
            }
            if (is_int($prev_close)) {
              $prev_close = number_format($prev_close, 2);
              $stock_results[$stock_symbol][stockapi_previous_close] = $prev_close;
            }
            else {
              $prev_close = number_format(floatval($prev_close), 2);
              $stock_results[$stock_symbol][stockapi_previous_close] = $prev_close;
            }
          }
        }
      }
      /*
       * As date code written date, most stock info in table td structure.
       * Switch statement in foreach assumes td elements in arr_tds is like:
       * <td class="key>52 week</td><td class="val">31.92 - 46.76</td>
       */
      foreach($arr_tds as $el) {
        switch ($el->getAttribute('class')) {
          case 'key' :
            $key_tmp = $el->nodeValue;
            $key_tmp = _stockapi_process_key($key_tmp);
            $key_orig = $key_tmp;
          break;
          case 'val' :
            $val_tmp = $el->nodeValue;
            if ($key_tmp == stockapi_vol_avg_for_postprocessing) {
              $volume = substr($val_tmp, 0, stripos($val_tmp, '/'));
              $volume = _stockapi_money_to_float($volume);
              $avg_vol = substr($val_tmp, stripos($val_tmp, '/')+1);
              $avg_vol = _stockapi_money_to_float($avg_vol);
              $stock_results[$stock_symbol][stockapi_volume] = $volume;
              $stock_results[$stock_symbol][stockapi_avg_daily_volume] = $avg_vol;
            }
            else if ($key_tmp == stockapi_market_cap) {
              // 6.95B becomes 6950000000.00
              $val_tmp = _stockapi_money_to_float($val_tmp);
              $stock_results[$stock_symbol][stockapi_market_cap] = $val_tmp;
            }
            else if ($key_tmp == stockapi_float_shares) {
              $val_tmp = _stockapi_money_to_float($val_tmp);
              $stock_results[$stock_symbol][stockapi_float_shares] = $val_tmp;
            }
            else if ($key_tmp == stockapi_pe_ratio) {
              $stock_results[$stock_symbol][stockapi_pe_ratio] = floatval($val_tmp);
            }
            else if ($key_tmp == stockapi_open) {
              $stock_results[$stock_symbol][stockapi_open] = floatval(trim($val_tmp));
            }
            else if(is_numeric($key_tmp)) {
              $stock_results[$stock_symbol][$key_tmp] = $val_tmp;
            }
          break;
        }
      }
      // The company name is in a section with ID 'appbar', search in there.
      $page_titles = $doc->getElementsByTagName('title');
      foreach ($page_titles as $page_title) {
        $title_text = trim($page_title->nodeValue);
        // Ex: <title>Some Company X Mines Inc: TSE:SCXM quotes &amp; new</title>
        $idx_quotes = stripos($title_text, ' quotes');
        $title_temp = substr($title_text, 0, $idx_quotes);
        $stock_results[$stock_symbol][stockapi_name] = $title_temp;
        if (stripos($title_text, 'NYSE Leveraged') === 0) {
          // Bug fix/workaround.
          $stock_results[$stock_symbol][stockapi_exchange] = 'NASDAQ';
        }
        if (!empty($title_temp)) {
          break;
        }
      }
      // chg and pct change.
      if (isset($stock_results[$stock_symbol][stockapi_chg]) &&
       isset($stock_results[$stock_symbol][stockapi_pct_chg])) {
        $chg = $stock_results[$stock_symbol][stockapi_chg];
        $pct = $stock_results[$stock_symbol][stockapi_pct_chg];
        if (strpos($chg, '-') === 0) {
          $pct = '(' . $pct . '%)';
        }
        else {
          $pct = '(+' . $pct . '%)';
        }
        if (!isset($stock_results[$stock_symbol][stockapi_chg_and_pct_chg])) {
          $stock_results[$stock_symbol][stockapi_chg_and_pct_chg] = $chg . ' ' . $pct;
        }
      }
      // Alternative scraping method is uncommented (below):
      //$node_values = $doc->getElementById('market-data-div');
      //$node_values = $node_values->nodeValue;
      // results are returned in row by row text format, need parsing.
      // $stock = _stockapi_parse_web($node_values, $stock_market_colon_symbol);
      // $stock_results = array_merge($stock, $stock_results);
    }
    //watchdog('stockapi_debug_parse_web', '<pre>@print_r</pre>', array('@print_r' => print_r( $stock, TRUE)));
    //watchdog('stockapi_results', '<pre>@print_r</pre>', array('@print_r' => print_r( $stock_results, TRUE)));
    return $stock_results;
  }
  else {
    $batches = array_chunk($symbols, $l);
    $stocks = array();
    foreach ($batches as $key => $batch) {
      $stocks = array_merge(stockapi_fetch($batch), $stocks);
    }
    return $stocks;
  }
}

/**
 * Generate Google Finance page URL.
 *
 * @param string $stock_market
 *   Stock market name, i.e. 'NASDAQ' or 'TSE'.
 * @param string $stock_symbol
 *   Stock symbol.
 *
 * @return string
 *   Google finance URL for given stock market and symbol.
 */
function stockapi_google_finance_url($stock_market, $stock_symbol) {
  $host = 'http://finance.google.ca/';
  $options = array(
    'query' => array(
      'q' => $stock_market . ':' . $stock_symbol,
    ),
  );

  drupal_alter('stockapi_google_finance_query', $host, $options, $stock_market, $stock_symbol);

  return url($host, $options);
}

function _stockapi_money_to_float($money_string) {
  $money_string = trim($money_string);
  $idx_of_scale = strlen($money_string);
  $idx_of_comma = strpos($money_string, ',');
  if (is_numeric($money_string)) {
    $money_float = floatval($money_string);
  }
  else {
    $scale = substr($money_string, -1);
    if ($idx_of_comma >= 0) {
      $money_string = str_replace(',', '', $money_string);
    }
    switch ($scale) {
      case 'M' :
        $money_float = floatval($money_string) * 1000000;
      break;
      case 'm' :
        $money_float = floatval($money_string) * 1000000;
      break;
      case 'B' :
        $money_float = floatval($money_string) * 1000000000;
      break;
      case 'b' :
        $money_float = floatval($money_string) * 1000000000;
      break;
      default:
        $money_float = floatval($money_string);
      break;
    }
  }
  return $money_float;
}

/*
 * Convert scraped key into the db schema key.
 */
function _stockapi_process_key($key) {

  $key = trim($key);
  switch ($key) {
    case 'Range' :
      $key = stockapi_days_range;
    break;
    case '52 week' :
      $key = stockapi_52_week_range;
    break;
    case 'Open' :
      $key = stockapi_open;
    break;
    // needs post processing, comes in as Vol / Avg.
    case 'Vol / Avg.' :
      $key = stockapi_vol_avg_for_postprocessing;
      // Split this later into stockapi_volume and stockapi_avg_daily_volume.
    break;
    case 'Mkt cap' :
      $key = stockapi_market_cap;
    break;
    case 'P/E' :
      $key = stockapi_pe_ratio;
    break;
    case 'Shares' :
      $key = stockapi_float_shares;
    break;
  }
  return $key;
}

/*
 * Convert scraped val into format expected by schema.
 */
function _stockapi_process_val($val) {
  return $val;
}

/*
 * Alternative scrape approach, grab nodeValue lines from whole section.
 */
function _stockapi_parse_web($value, $corp) {
  $sans_vide = preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n", $value);
  //watchdog('stockapi_sans_vide', '<pre>@print_r</pre>', array('@print_r' => print_r( $sans_vide, TRUE)));

  $array = explode("\n", $sans_vide);
  $stock = array();
  $stock[$corp][0]= $corp;//symbol
  foreach ($array as $key => $value) {
    $date_temp = '';
    if ($key == 1) {
      $stock[$corp][$key] = $value; // trade_price_only
    }
    if ($key == 2) {
      $stock[$corp][$key] = substr($value, (stripos($value, '+')+1)); //chg
    }
    if ($key == 3) {
      $stock[$corp][$key] = $value; //pct_chg
    }
    if ($key == 4) {
      $date_temp = substr($value, 0, stripos($value, ' -'));
      $stock[$corp][$key] = strtotime($date_temp . ' 2017'); // Updated
    }
    if (stripos($array[$key], 'Close') > 0) {
      $stock[$corp][5] = $array[1];// previous-close
    }
    if (stripos($array[$key], 'ange') == 1) {
      $stock[$corp][6] = $array[$key+1];// days_range
    }
    if (stripos($array[$key], '2 week') == 1) {
      $stock[$corp][7] = $array[$key+1];// 52_week_range
    }
    if (stripos($array[$key], 'pen') == 1) {
      $stock[$corp][8] = $array[$key+1];// open
    }
    if (stripos($array[$key], 'ol / Avg.') == 1) {
      $val = $array[$key+1];
      $stock[$corp][9] = substr($val, 0, (stripos($val, '/')));// volume
      $stock[$corp][10] = substr($val, (stripos($val, '/')+1));// avg_daily_volume
    }
    if (stripos($array[$key], 'kt cap') == 1) {
      $stock[$corp][11] = $array[$key+1];// market_cap
    }
    if (!empty($date_temp)) {
      $stock[$corp][12] = $date_temp; // trade_date string
      $stock[$corp][13] = $date_temp; // last_trade_date string
    }

  }
  return $stock;
}
/**
 * Save a single stock quote to the database.
 *
 * @param $stock object
 *   The stock object keys' must match the field names of the database table.
 *   The stock object values' are the data to insert.
 *
 * @return
 *   TRUE on success, FALSE on failure
 */
function stockapi_save($stock) {
  // Fix potential text in float fields.
  $stock = _stockapi_fix_floats($stock);

  if (isset($stock->symbol) && db_query("SELECT symbol FROM {stockapi} WHERE symbol = :symbol", array(':symbol' => $stock->symbol))->fetchField()) {
    // Update
    $symbol = $stock->symbol;
    $stock = (array) $stock;
    unset($stock['symbol']); // Primary key

    $updated = db_update('stockapi')
        ->fields((array) $stock)
        ->condition('symbol', $symbol)
        ->execute();

    return $updated;
  }
  else {
    //Insert
    $inserted = db_insert('stockapi')
        ->fields((array) $stock)
        ->execute();

    return $inserted;
  }
}

/**
 * Return ticker information for a single symbol from our database, not Google.
 *
 * @param $symbol string
 *   The ticker symbol to retrieve information for.
 * @param $q string
 *   The type of information to retrieve. Can be either: basic, extended,
 *   or realtime. The default is the site-wide setting, which is usually the
 *   best bet here.
 * @return object
 */
function stockapi_load($symbol, $q = NULL) {
  static $stocks = array();

  if (!drupal_strlen($symbol)) {
    return array();
  }

  if (!isset($stocks[$symbol])) {
    $fields = _stockapi_get_fields($q);
    $results = db_query("SELECT " . implode(',', $fields) . " FROM {stockapi} WHERE symbol = :symbol", array(':symbol' => $symbol), array('fetch' => PDO::FETCH_ASSOC));
    if (count($results)) {
      foreach ($results as $result) {
        $stocks[$symbol] = $result;
      }
    }
  }
  else {
    $stocks[$symbol] = array();
  }

  return $stocks[$symbol];
}

/**
 * Return ticker information for a batch of symbols from our database, not Google.
 *
 * @param $symbol array
 *   An array of ticker symbols to retrieve information for.
 * @param $q string
 *   The type of information to retrieve. Can be either: basic, extended,
 *   or realtime. The default is the site-wide setting, which is usually the
 *   best bet here.
 * @return array
 *   An array of stocks, keyed by their ticker symbol. Each stock is an object within the array.
 */
function stockapi_multiload($symbol, $q = NULL) {
  $stocks = array();
  $fields = _stockapi_get_fields($q);
  $symbols = implode(', ', array_map('_stockapi_quote_it', $symbol));
  $results = db_query("SELECT " . implode(', ', $fields) . " FROM {stockapi} WHERE symbol IN ({$symbols})");
  foreach ($results as $stock) {
    $stocks[$stock->symbol] = $stock;
  }

  return $stocks;
}

/**
 * Wrap a string in quotes. Usually used with array_map().
 */
function _stockapi_quote_it($string, $style = 'single') {
  return ($style != 'single') ? '"' . $string . '"' : "'" . $string . "'";
}

/**
 * Return the field parameter string to pull the correct columns from Google.
 */
function stockapi_get_quotetype($q = NULL) {
  $quotetype = array();
  $quotetype['basic'] = 'snl1d1t1c1p2va2bapomwerr1dyj1x';
  $quotetype['extended'] = $quotetype['basic'] . 'cs7t8e7e8e9r6r7r5b4p6p5j4m3m4k3t7it6j5i5c8r2g6g5w4v7n4l2l3g3d2g4g1w1v1c3p1s1m6m5m8m7qk5k4j6';
  $quotetype['realtime'] = $quotetype['basic'] . 'b2b3k2k1c6m2j3';

  if ($quotetype[$q]) {
    return $quotetype[$q];
  }
  // Always return the basic set as a fall through.
  return $quotetype['basic'];
}

/**
 * Convert a stock from Google to its object oriented counterpart.
 *
 * @return object
 */
function _stockapi_to_object($stock, $q = NULL) {
  $fields = _stockapi_get_fields($q);
  $ns = count($fields);
  if ($ns) {
    $s = new stdClass();
    for ($i = 0; $i < $ns; $i++) {
      $s->{$fields[$i]} = $stock[$i];
      if (isset($stock[$i])) {
        $s->{$fields[$i]} = $stock[$i];
      }
      else {
        $s->{$fields[$i]} = NULL;
        watchdog('stockapi_info', '<pre>@print_r</pre>', array('@print_r'
         => print_r( "Symbol " . $stock[0] . " Element $i NULL", TRUE))
        );
      }
    }
  }
  $s->updated = REQUEST_TIME;

  return $s;
}

/**
 * Given the current site-wide quotetype, generate the actual database fields
 * names currently active. These field names usually double as the keys for the
 * stock object.
 */
function _stockapi_get_fields($q = NULL) {
  if (!$q) {
    $q = variable_get('stockapi_quotetype', 'basic');
  }
  preg_match_all('/[a-z][0-9]?/', stockapi_get_quotetype($q), $yfields);
  $yfields = $yfields[0];

  $fields = explode(', ', 'symbol, last_trade_price_only, chg, pct_chg, updated, previous_close, days_range, 52_week_range, open, volume, avg_daily_volume, market_cap, trade_date, last_trade_date, pe_ratio, float_shares, chg_and_pct_chg, last_trade_time, exchange, name');
  //watchdog('stockapi_fields_debug', '<pre>@print_r</pre>', array('@print_r' => print_r( $fields, TRUE)));
  return $fields;
}

/**
 * Internal helper function to deal cleanly with various HTTP response codes.
 */
function _stockapi_request_failure($results) {
  switch ($results->code) {
    case '200': // Success!
    case '304': // Not modified, nothing to do.
      return FALSE;
    default:
      watchdog('stockapi', 'Failed to retrieve stock quotes with error: %error', array('%error' => $results->error));
      return TRUE;
  }
  return FALSE;
}

/**
 * Internal helper function to change text values returned for float
 * fields to '0'.
 */
function _stockapi_fix_floats($stock) {
  $stock = (array) $stock;
  $schema = drupal_get_schema('stockapi');

  foreach ($schema['fields'] as $name => $field) {
    if ($field['type'] == 'float' && isset($stock[$name]) && !is_numeric($stock[$name])) {
      $stock[$name] = 0;
    }
  }

  return (object) $stock;
}

